package dev.fyloz.trial.colorrecipesexplorer.service

import dev.fyloz.trial.colorrecipesexplorer.model.Material
import dev.fyloz.trial.colorrecipesexplorer.model.Mix
import dev.fyloz.trial.colorrecipesexplorer.model.MixMaterial
import dev.fyloz.trial.colorrecipesexplorer.model.mixMaterial
import dev.fyloz.trial.colorrecipesexplorer.repository.MixMaterialRepository
import org.springframework.context.annotation.Lazy
import org.springframework.stereotype.Service

interface MixMaterialService : ModelService<MixMaterial, MixMaterialRepository> {
    /** Checks if one or more mix materials have the given [material]. */
    fun existsByMaterial(material: Material): Boolean

    /** Creates [MixMaterial]s from the given [map]. The [map] must have the format <Material ID, Quantity>. */
    fun createFromMap(mix: Mix, map: Map<Long, Float>): Collection<MixMaterial>

    /** Creates a [MixMaterial] with the material with the given [materialId] and the given [quantity]. */
    fun create(mix: Mix, materialId: Long, quantity: Float): MixMaterial

    /** Updates the [quantity] of the given [mixMaterial]. */
    fun updateQuantity(mixMaterial: MixMaterial, quantity: Float): MixMaterial
}

@Service
class MixMaterialServiceImpl(
    mixMaterialRepository: MixMaterialRepository,
    @Lazy val materialService: MaterialService
) : AbstractModelService<MixMaterial, MixMaterialRepository>(mixMaterialRepository), MixMaterialService {
    override fun existsByMaterial(material: Material): Boolean = repository.existsByMaterial(material)
    override fun createFromMap(mix: Mix, map: Map<Long, Float>): Collection<MixMaterial> =
        map.map { create(mix, it.key, it.value) }

    override fun create(mix: Mix, materialId: Long, quantity: Float): MixMaterial =
        mixMaterial(mix = mix, material = materialService.getById(materialId), quantity = quantity)

    override fun updateQuantity(mixMaterial: MixMaterial, quantity: Float) =
        update(mixMaterial.apply {
            this.quantity = quantity
        })
}
